﻿Public Class Form1
    ' The loaded map image.
    Private Map As Bitmap = Nothing

    ' Known units.
    Private Enum Units
        Undefined
        Miles
        Yards
        Feet
        Kilometers
        Meters
    End Enum

    ' Key map values.
    Private ScaleDistanceInUnits As Double = -1
    Private ScaleDistanceInPixels As Double = -1
    Private CurrentUnit As Units = Units.Miles
    Private CurrentDistance As Double = -1

    ' Open a map image.
    Private Sub mnuFileOpen_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuFileOpen.Click
        If (ofdMap.ShowDialog() = DialogResult.OK) Then
            Map = LoadUnlocked(ofdMap.FileName)
            picMap.Image = Map
        End If

        btnScale.Enabled = True
        btnDistance.Enabled = True
        CurrentUnit = Units.Undefined
        DisplayValues()
    End Sub

    Private Sub mnuFileExit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuFileExit.Click
        Me.Close()
    End Sub

    ' Set the desired units.
    Private Sub btnUnits_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnYards.Click, btnMiles.Click, btnMeters.Click, btnKilometers.Click, btnFeet.Click
        ' Find a factor to convert from the old units to meters.
        Dim conversion As Double = 1
        If (CurrentUnit = Units.Feet) Then
            conversion = 0.3048
        ElseIf (CurrentUnit = Units.Yards) Then
            conversion = 0.9144
        ElseIf (CurrentUnit = Units.Miles) Then
            conversion = 1609.344
        ElseIf (CurrentUnit = Units.Kilometers) Then
            conversion = 1000
        End If

        Dim menu_item As ToolStripMenuItem = DirectCast(sender, ToolStripMenuItem)
        Select menu_item.Text
            Case "Miles"
                CurrentUnit = Units.Miles
            Case "Yards"
                CurrentUnit = Units.Yards
            Case "Feet"
                CurrentUnit = Units.Feet
            Case "Kilometers"
                CurrentUnit = Units.Kilometers
            Case "Meters"
                CurrentUnit = Units.Meters
        End Select
        btnUnits.Text = CurrentUnit.ToString()

        ' Find a factor to convert from meters to the new units.
        If (CurrentUnit = Units.Feet) Then
            conversion *= 3.28083
        ElseIf (CurrentUnit = Units.Yards) Then
            conversion *= 1.09361
        ElseIf (CurrentUnit = Units.Miles) Then
            conversion *= 0.000621
        ElseIf (CurrentUnit = Units.Kilometers) Then
            conversion *= 0.001
        End If

        ' Convert and display the values.
        ScaleDistanceInUnits *= conversion
        CurrentDistance *= conversion
        DisplayValues()
    End Sub

    ' Load the image into a Bitmap, clone it, and
    ' set the PictureBox's Image property to the Bitmap.
    Private Function LoadUnlocked(ByVal file_name As String) As Bitmap
        Using bm As New Bitmap(file_name)
            Dim result As New Bitmap(bm.Width, bm.Height)
            Using gr As Graphics = Graphics.FromImage(result)
                gr.DrawImage(bm, 0, 0)
            End Using
            Return result
        End Using
    End Function

    ' Display the units per pixels and the measured distance.
    Private Sub DisplayValues()
        If (CurrentUnit = Units.Undefined) Then
            btnScale.Text = "<Click to set scale>"
        Else
            Dim units As Double = ScaleDistanceInUnits / ScaleDistanceInPixels
            btnScale.Text = String.Format("{0} {1}/pixel", _
                units.ToSignificantDigits(3), _
                CurrentUnit.ToString())
        End If

        If (CurrentDistance < 0) Then
            btnDistance.Text = "<Click to measure a distance>"
        Else
            btnDistance.Text = String.Format("Distance: {0} {1}", _
                CurrentDistance.ToSignificantDigits(3), CurrentUnit)
        End If
    End Sub

    ' Reset the scale.
    Private StartPoint, EndPoint As Point
    Private Sub btnScale_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnScale.Click
        lblInstructions.Text = "Click and drag from the start and end point of the map's scale bar."
        picMap.Cursor = Cursors.Cross

        AddHandler picMap.MouseDown, AddressOf Scale_MouseDown
    End Sub
    Private Sub Scale_MouseDown(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs)
        StartPoint = e.Location
        RemoveHandler picMap.MouseDown, AddressOf Scale_MouseDown
        AddHandler picMap.MouseMove, AddressOf Scale_MouseMove
        AddHandler picMap.MouseUp, AddressOf Scale_Mouseup
    End Sub
    Private Sub Scale_MouseMove(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs)
        EndPoint = e.Location
        DisplayScaleLine()
    End Sub
    Private Sub Scale_MouseUp(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs)
        RemoveHandler picMap.MouseMove, AddressOf Scale_MouseMove
        RemoveHandler picMap.MouseUp, AddressOf Scale_MouseUp
        picMap.Cursor = Cursors.Default
        lblInstructions.Text = ""

        ' Get the scale.
        Dim dlg As New ScaleDialog()
        If (dlg.ShowDialog() = DialogResult.OK) Then
            ' Get the distance on the screen.
            Dim dx As Integer = EndPoint.X - StartPoint.X
            Dim dy As Integer = EndPoint.Y - StartPoint.Y
            Dim dist As Double = Math.Sqrt(dx * dx + dy * dy)
            If (dist < 1) Then Return
            ScaleDistanceInPixels = dist

            ' Parse the distance.
            ParseDistanceString(dlg.txtScaleLength.Text, _
                ScaleDistanceInUnits, CurrentUnit)

            ' Display the units.
            btnUnits.Text = CurrentUnit.ToString()

            ' Display the scale and measured distance.
            CurrentDistance = -1
            DisplayValues()
        End If
    End Sub

    ' Display the scale line.
    Private Sub DisplayScaleLine()
        Dim bm As Bitmap = DirectCast(Map.Clone(), Bitmap)
        Using gr As Graphics = Graphics.FromImage(bm)
            gr.DrawLine(Pens.Red, StartPoint, EndPoint)
        End Using
        picMap.Image = bm
    End Sub

    ' Parse a distance string. Return the length in meters.
    Private Sub ParseDistanceString(ByVal txt As String, ByRef distance As Double, ByRef unit As Units)
        txt = txt.Trim()

        ' Find the longest substring that makes sense as a double.
        Dim i As Integer = DoublePrefixLength(txt)
        If (i <= 0) Then
            distance = -1
            unit = Units.Undefined
        Else
            ' Get the distance.
            distance = Double.Parse(txt.Substring(0, i))

            ' Get the unit.
            Dim unit_string As String = txt.Substring(i).Trim().ToLower()
            If (unit_string.StartsWith("mi")) Then
                unit = Units.Miles
            ElseIf (unit_string.StartsWith("y")) Then
                unit = Units.Yards
            ElseIf (unit_string.StartsWith("f")) Then
                unit = Units.Feet
            ElseIf (unit_string.StartsWith("'")) Then
                unit = Units.Feet
            ElseIf (unit_string.StartsWith("k")) Then
                unit = Units.Kilometers
            ElseIf (unit_string.StartsWith("m")) Then
                unit = Units.Meters
            Else
                unit = Units.Undefined
            End If
        End If
    End Sub

    ' Return the length of the longest prefix
    ' string that makes sense as a double.
    Private Function DoublePrefixLength(ByVal txt As String) As Integer
        For i As Integer = 1 To txt.Length
            Dim test_string As String = txt.Substring(0, i)
            Dim test_value As Double
            If (Not Double.TryParse(test_string, test_value)) Then Return i - 1
        Next i
        Return txt.Length
    End Function

    ' Let the user draw something and calculate its length.
    Private Sub btnDistance_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnDistance.Click
        lblInstructions.Text = "Click and draw to define the path that you want to measure."
        picMap.Cursor = Cursors.Cross

        DistancePoints = New List(Of Point)()
        AddHandler picMap.MouseDown, AddressOf Distance_MouseDown
    End Sub

    Private DistancePoints As List(Of Point)
    Private Sub Distance_MouseDown(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs)
        DistancePoints.Add(e.Location)
        RemoveHandler picMap.MouseDown, AddressOf Distance_MouseDown
        AddHandler picMap.MouseMove, AddressOf Distance_MouseMove
        AddHandler picMap.MouseUp, AddressOf Distance_MouseUp
    End Sub
    Private Sub Distance_MouseMove(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs)
        DistancePoints.Add(e.Location)
        DisplayDistanceCurve()
    End Sub
    Private Sub Distance_MouseUp(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs)
        RemoveHandler picMap.MouseMove, AddressOf Distance_MouseMove
        RemoveHandler picMap.MouseUp, AddressOf Distance_MouseUp
        picMap.Cursor = Cursors.Default
        lblInstructions.Text = ""

        ' Measure the curve.
        Dim distance As Double = 0
        For i As Integer = 1 To DistancePoints.Count - 1
            Dim dx As Integer = DistancePoints(i).X - DistancePoints(i - 1).X
            Dim dy As Integer = DistancePoints(i).Y - DistancePoints(i - 1).Y
            distance += Math.Sqrt(dx * dx + dy * dy)
        Next i

        ' Convert into the proper units.
        CurrentDistance = distance * ScaleDistanceInUnits / ScaleDistanceInPixels

        ' Display the result.
        DisplayValues()
    End Sub

    ' Display the distance curve.
    Private Sub DisplayDistanceCurve()
        Dim bm As Bitmap = DirectCast(Map.Clone(), Bitmap)
        Using gr As Graphics = Graphics.FromImage(bm)
            gr.DrawLines(Pens.Red, DistancePoints.ToArray())
        End Using
        picMap.Image = bm
    End Sub
End Class
